@@ -8,32 +8,6 @@ from docutils.parsers import rst
 from docutils.parsers.rst import directives
 
 
-def change_pathto(app, pagename, templatename, context, doctree):
-    pathto = context.get('pathto')
-
-    def gh_pathto(otheruri, *args, **kw):
-        if otheruri.startswith('_'):
-            otheruri = otheruri[1:]
-        return pathto(otheruri, *args, **kw)
-    context['pathto'] = gh_pathto
-
-
-def move_private_folders(app, e):
-    def join(dir, *args):
-        return os.path.join(app.builder.outdir, dir, *args)
-
-    for item in os.listdir(app.builder.outdir):
-        if item.startswith('_') and os.path.isdir(join(item)):
-            if item == '_modules':
-                continue
-            if os.path.isdir(join(item[1:])):
-                shutil.rmtree(join(item[1:]))
-            if item == '_static':
-                for dirname in glob.glob(join(item, '*', '.git')):
-                    shutil.rmtree(dirname)
-            shutil.move(join(item), join(item[1:]))
-
-
 def hide_title(argument):
     return directives.choice(argument, ('true', 'false'))
 
@@ -84,14 +58,6 @@ class Step(rst.Directive):
                    'data-rotate-z': directives.unchanged,
                    }
 
-    def resolve_func(self, name):
-        if hasattr(funcs, name):
-            return getattr(funcs, name)
-        else:
-            mod, func = name.split('.')
-            mod = __import__(mod, globals(), locals(), [''])
-            return getattr(mod, func)
-
     def run(self):
         if 'reset' in os.environ:
             del os.environ['reset']
@@ -99,6 +65,7 @@ class Step(rst.Directive):
             Step.last_coord = {}
 
         parent = self.state.parent
+
         source = parent.document.attributes['source']
         global_options = Impress.opts.setdefault(source, {})
         for k, v in global_options.items():
@@ -114,20 +81,6 @@ class Step(rst.Directive):
             if self.options.get('hide-title', 'false') == 'true':
                 title = parent.next_node()
                 title.attributes['classes'].insert(0, 'hidden')
-            if 'data-x' not in self.options:
-                func = self.resolve_func(self.options.get('func', 'default'))
-                amount = Step.amounts.setdefault(source, 0)
-                last_coord = Step.last_coord.setdefault(source, {})
-                new_attrs = func(self, amount, last_coord)
-                for k, v in new_attrs.items():
-                    last_coord[k] = v
-                    if k in ('x', 'y', 'z',
-                             'rotate_x', 'rotate_y', 'rotate_z',
-                             'scale'):
-                        k = 'data-%s' % k.replace('_', '-')
-                        if k not in self.options:
-                            attrs[k] = [str(v)]
-                Step.amounts[source] += 1
         else:
             print('%s:: WARNING: %s found out of section are ignored' % (
                              source, self.__class__.__name__.lower()))
@@ -145,9 +98,72 @@ class Slide(Step):
         return super(Slide, self).run()
 
 
+def resolve_func(name):
+    if hasattr(funcs, name):
+        return getattr(funcs, name)
+    else:
+        mod, func = name.split('.')
+        try:
+            mod = __import__(mod, globals(), locals(), [''])
+            return getattr(mod, func)
+        except (ImportError, AttributeError):
+            raise ImportError("Could not resolve %s" % name)
+
+
+def slides_position(app, doctree, docname):
+    """doctree hook to reorganise slides **after** toctrees are build"""
+    slides = [s for s in doctree.children if s.tagname == 'section']
+    for slide in slides:
+        slides.extend([s for s in slide.children if s.tagname == 'section'])
+        slide.children = [s for s in slide.children if s.tagname != 'section']
+    doctree.children = [s for s in doctree.children if s.tagname != 'section']
+    doctree.children.extend(slides)
+    slides = [s for s in doctree.children if s.tagname == 'section']
+    for slide in slides:
+        if 'step' not in slide.attributes['classes']:
+            slide.attributes['classes'].extend(['step', 'slide'])
+    slides = [funcs.Slide(i, s) for i, s in enumerate(slides)]
+    previous = None
+    for i, coord in enumerate(slides):
+        slide = coord.section
+        if previous:
+            coord.update(previous)
+        func = resolve_func(slide.attributes.get('func', 'default'))
+        if func(coord, slides) is True:
+            break
+        previous = coord
+
+
+def change_pathto(app, pagename, templatename, context, doctree):
+    pathto = context.get('pathto')
+
+    def gh_pathto(otheruri, *args, **kw):
+        if otheruri.startswith('_'):
+            otheruri = otheruri[1:]
+        return pathto(otheruri, *args, **kw)
+    context['pathto'] = gh_pathto
+
+
+def move_private_folders(app, e):
+    def join(dir, *args):
+        return os.path.join(app.builder.outdir, dir, *args)
+
+    for item in os.listdir(app.builder.outdir):
+        if item.startswith('_') and os.path.isdir(join(item)):
+            if item == '_modules':
+                continue
+            if os.path.isdir(join(item[1:])):
+                shutil.rmtree(join(item[1:]))
+            if item == '_static':
+                for dirname in glob.glob(join(item, '*', '.git')):
+                    shutil.rmtree(dirname)
+            shutil.move(join(item), join(item[1:]))
+
+
 def setup(app):
     app.add_directive('impress', Impress)
     app.add_directive('step', Step)
     app.add_directive('slide', Slide)
+    app.connect('doctree-resolved', slides_position)
     app.connect('html-page-context', change_pathto)
     app.connect('build-finished', move_private_folders)
